// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/LCD-Ansteuerung
//
// Die Pinbelegung ist ber defines in lcd-routines.h einstellbar
 
#include <avr/io.h>
#include "lcd-routines.h"
#include <util/delay.h>
#include "global.h"
#include <stdlib.h>
#include "debounce.h"
#include "THC.h"
 

 /*
 
Funktion zur Umwandlung einer vorzeichenlosen 32 Bit Zahl in einen String
 
*/
 
void my_uitoa(uint32_t zahl, char* string) {
  int8_t i;                             // schleifenzhler
 
  string[10]='\0';                       // String Terminator
  for(i=9; i>=0; i--) {
    string[i]=(zahl % 10) +'0';         // Modulo rechnen, dann den ASCII-Code von '0' addieren
    zahl /= 10;
  }
}

void my_itoa(int32_t zahl, char* string) {
  uint8_t i;
 
  string[11]='\0';                  // String Terminator
  if( zahl < 0 ) {                  // ist die Zahl negativ?
    string[0] = '-';              
    zahl = -zahl;
  }
  else string[0] = ' ';             // Zahl ist positiv
 
  for(i=10; i>=1; i--) {
    string[i]=(zahl % 10) +'0';     // Modulo rechnen, dann den ASCII-Code von '0' addieren
    zahl /= 10;
  }
}
// sendet ein Datenbyte an das LCD
 
void lcd_data(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_PORT |= (1<<LCD_RS);        // RS auf 1 setzen
 
   temp1 = temp1 >> 4;
   temp1 = temp1 & 0x0F;
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp1;               // setzen
   lcd_enable();
 
   temp2 = temp2 & 0x0F;
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp2;               // setzen
   lcd_enable();
   
   _delay_us(42);
}
 
// sendet einen Befehl an das LCD
 
void lcd_command(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_PORT &= ~(1<<LCD_RS);        // RS auf 0 setzen
 
   temp1 = temp1 >> 4;              // oberes Nibble holen
   temp1 = temp1 & 0x0F;            // maskieren
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp1;               // setzen
   lcd_enable();
 
   temp2 = temp2 & 0x0F;            // unteres Nibble holen und maskieren
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp2;               // setzen
   lcd_enable();
   
   _delay_us(42);
}
 
// erzeugt den Enable-Puls
void lcd_enable(void)
{
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers einfgen
   // http://www.mikrocontroller.net/topic/81974#685882
   LCD_PORT |= (1<<LCD_EN);
    _delay_us(1);                   // kurze Pause
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers verlngern
   // http://www.mikrocontroller.net/topic/80900
   LCD_PORT &= ~(1<<LCD_EN);
}
 
// Initialisierung: 
// Muss ganz am Anfang des Programms aufgerufen werden.
 
void lcd_init(void)
{
   LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) | (1<<LCD_EN);   // Port auf Ausgang schalten
 
   // muss 3mal hintereinander gesendet werden zur Initialisierung
 
   _delay_ms(15);
   LCD_PORT &= 0xF0;
   LCD_PORT |= 0x03;            
   LCD_PORT &= ~(1<<LCD_RS);      // RS auf 0
   lcd_enable();
 
   _delay_ms(5);
   lcd_enable();
 
   _delay_ms(1);
   lcd_enable();
   _delay_ms(1);
 
   // 4 Bit Modus aktivieren 
   LCD_PORT &= 0xF0;
   LCD_PORT |= 0x02;
   lcd_enable();
   _delay_ms(1);
 
   // 4Bit / 2 Zeilen / 5x7
   lcd_command(0x28);
    
   // Display ein / Cursor aus / kein Blinken
   lcd_command(0x0C); 
 
   // inkrement / kein Scrollen
   lcd_command(0x06);
 
   lcd_clear();
}
 
// Sendet den Befehl zur Lschung des Displays
 
void lcd_clear(void)
{
   lcd_command(CLEAR_DISPLAY);
   _delay_ms(5);
}
 
// Sendet den Befehl: Cursor Home
 
void lcd_home(void)
{
   lcd_command(CURSOR_HOME);
   _delay_ms(5);
}
 
// setzt den Cursor in Zeile y (1..4) Spalte x (0..15)
 
void set_cursor(uint8_t x, uint8_t y)
{
  uint8_t tmp;
  _delay_ms(42);
  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    case 3: tmp=0x80+0x14+x; break;    // 3. Zeile
    case 4: tmp=0x80+0x54+x; break;    // 4. Zeile
    default: return;                   // fr den Fall einer falschen Zeile
  }
  lcd_command(tmp);
}
 
// Schreibt einen String auf das LCD
 
void lcd_string(char *data)
{
    while(*data) {
        lcd_data(*data);
        data++;
    }
}

void my_print_LCD(char* string, uint8_t start, uint8_t komma, uint8_t frac) {
 
  uint8_t i;            // Zhler
  uint8_t flag=0;       // Merker fr fhrende Nullen
 
  // Vorzeichen ausgeben  
  if (string[0]=='-') lcd_data('-'); else ;//lcd_data(' ');
 
  // Vorkommastellen ohne fhrende Nullen ausgeben
  for(i=start; i<komma; i++) {
    if (flag==1 || string[i]!='0') {
      lcd_data(string[i]);
      flag = 1;
    }
    else lcd_data(' ');         // Leerzeichen oder 0
  }
 
  lcd_data(',');                // Komma ausgeben
 
  // Nachkommastellen ausgeben
  for(; i<(komma+frac); i++) lcd_data(string[i]);
 
}


void lcd_menue(unsigned char i)


{      

		switch (i) 

		{ 	case 1: 
					set_cursor(0,1);
					lcd_string( "----THC Controll----" );	
					set_cursor(0,2);
					lcd_string( "Set value:" );	
					set_cursor(19,2);
					lcd_string( "V" );
					
					char Buffer[10]; // in diesem {} lokal
					itoa( voltage_soll, Buffer,10 ); 
					char Buffer3[10]; // in diesem {} lokal
					itoa( voltage_ist, Buffer3, 10); 
					
					
					
					if (voltage_soll >= 1000)
					{
					set_cursor(13,2);
					my_print_LCD(Buffer,0,3,1);		
					}
					else{	
					set_cursor(13,2);
					lcd_string( " " );
					set_cursor(14,2);
					my_print_LCD(Buffer,0,2,1);
					}					
					
					 
					
					 
					 set_cursor(0,3);
					 lcd_string( "Actual val.:" );	
					set_cursor(19,3);
					lcd_string( "V" );
					
					
					if (voltage_ist <= 500)
					{
						set_cursor(13,3);
						lcd_string( "  OFF" );
											
					}
					
					
					else if (voltage_ist >= 1000)
					{
					set_cursor(13,3);
					my_print_LCD(Buffer3,0,3,1);		
					}
					else{	
					set_cursor(13,3);
					lcd_string( " " );
					set_cursor(14,3);
					my_print_LCD(Buffer3,0,2,1);
					}			
							
					
								 
					 
 

			break;

			case 2:	
				
					set_cursor(0,1);
					lcd_string( "------Voltage-------" );	
					set_cursor(0,2);
					lcd_string( "Hysterese:" );
					set_cursor(17,2);
					lcd_string( "V" );
					
					char Buffer21[10]; // in diesem {} lokal
					itoa( hysterese, Buffer21, 10); 
			
					if (hysterese <= 9)
					 { set_cursor(14,2);
						 lcd_string( "0" );
						set_cursor(15,2);
					   my_print_LCD(Buffer21,0,0,1);	
					 }
					 else
					 {set_cursor(14,2);
					  my_print_LCD(Buffer21,0,1,1);
					 }						
					
					
					
					
					if (button3() )
					{hysterese++;
					}
					if (button2())
					{hysterese--;
						if (hysterese < 0)
						{hysterese=0;
						}
					}
			
			break;
			
			case 3:	
					set_cursor(0,1);
					lcd_string( "-------Arc OK-------" );	
					set_cursor(0,2);
					lcd_string( "Set OK value:" );	
					set_cursor(0,3);
					lcd_string( "Actual value:" );
					set_cursor(17,2);
					lcd_string( "A" );
					set_cursor(17,3);
					lcd_string( "A" );
					
					if (button3() )
					{arc_soll++;
					}
					if (button2())
					{arc_soll--;
						if (arc_soll < 0)
						{arc_soll=0;
						}
					}
					
					char Buffer4[20]; // in diesem {} lokal
					itoa( arc_soll, Buffer4, 10 ); 
					char Buffer5[20]; // in diesem {} lokal
					itoa( arc_ist, Buffer5, 10 ); 
					
					
					if (arc_soll <= 9)
					 { set_cursor(15,2);
					    lcd_string( Buffer4 );
						set_cursor(14,2);
						lcd_string( " " );
					 }
					 else
					 {set_cursor(14,2);
					  lcd_string( Buffer4 );
					 }		
					 
					 if (arc_ist <= 9)
					 { set_cursor(15,3);
					    lcd_string( Buffer5 );
						set_cursor(14,3);
						lcd_string( " " );
					 }
					 else
					 {set_cursor(14,3);
					  lcd_string( Buffer5 );
					 }					


			break;
			
		}
}		